@@ -4,14 +4,15 @@ import time
 import logging
 from pythonagent.agent.internal import udp
 from pythonagent.agent.probes.havoc.havoc_constants import HAVOC_TYPES, EXPIRE_DURATION_MS, SIZE_OF_OBJECTS_BYTES, HAVOC_PROFILES_JSON
-
+from pythonagent.agent.probes.havoc.custom_memory_stress import apply_memory_leak
 havoc_types = {
     1: "InBoundService_Delay",
     2: "InBoundService_Failure",
     3: "OutBoundService_Delay",
     4: "OutBoundService_Failure",
     5: "MethodCall_Delay",
-    6: "MethodCall_Failure"
+    6: "MethodCall_Failure",
+    7: "Custom_Memory_Leak"
 }
 
 class NDHavocException(Exception):
@@ -23,6 +24,8 @@ class NDHavocException(Exception):
 class NDNetHavocRequest(object):
     def __init__(self, havoc_conf, header_dict):
 
+        self.logger = logging.getLogger('pythonagent.agent')
+
         if hasattr(havoc_conf, 'havocType'):
             self.havocType = havoc_conf.havocType
 
@@ -41,22 +44,23 @@ class NDNetHavocRequest(object):
             self.hostname = havoc_conf.hostname
 
         self.startTime = time.time_ns() // 1000000
+        self.applying = False
         self.header_dict = header_dict
         self.totalDurationInSec = havoc_conf.totalDurationInSec
 
         try:
+            self.memoryLeaksInSec = havoc_conf.memoryLeaksInSec
+            self.leaksINMB = havoc_conf.leaksINMB
             self.enable = havoc_conf.enable
             self.bTName = havoc_conf.bTName
             self.isHavocEnable = False
             self.threshOld = havoc_conf.threshOld
             self.protocol = havoc_conf.protocol
-            self.leaksINMB = havoc_conf.leaksINMB
-            self.memoryLeaksInSec = havoc_conf.memoryLeaksInSec
+            #self.memoryLeaksInSec = havoc_conf.memoryLeaksInSec
             self.backendName = havoc_conf.backendName
 
         except Exception as e:
-            print(e)
-            pass
+            self.logger.warning(e)
 
 
 
@@ -86,7 +90,8 @@ class NDNetHavocMonitor(object):
                 "OutBoundService_Delay": [],
                 "OutBoundService_Failure": [],
                 "MethodCall_Delay": [],
-                "MethodCall_Failure": []
+                "MethodCall_Failure": [],
+                "Custom_Memory_Leak": []
             }
 
             self.is_parsed = False
@@ -103,7 +108,6 @@ class NDNetHavocMonitor(object):
             raise e
 
     def validate_config(self, havoc_conf):
-        print("\n\n\n VALIDATING \n\n\n")
         self.logger.info("Validating config")
         is_valid = False
 
@@ -218,6 +222,18 @@ class NDNetHavocMonitor(object):
         else:
             self.logger.debug("delayInSec not needed")
 
+        if havoc_type in [7]:
+            if not hasattr(havoc_conf, "leaksINMB"):
+                return_message = "leaksINMB not found for custom_memory_leak"
+                return is_valid, return_message
+            else:
+                if not hasattr(havoc_conf, "memoryLeaksInSec"):
+                    return_message = "memoryLeaksInSec not found for custom_memory_leak"
+                    return is_valid, return_message
+                else:
+                    self.logger.debug("memoryLeaksInSec validated")
+        else:
+            self.logger.debug("leaksINMB, memoryLeaksInSec not needed")
         # 1.InBoundServicdelayhavoc
         # 2.InBoundServicefailhavoc
         # 3.OutBoundServicedelayhavoc
@@ -231,6 +247,73 @@ class NDNetHavocMonitor(object):
         return_message = "Config Validated"
         return is_valid, return_message
 
+    def validate_config_new(self, havoc_conf):
+        self.logger.info("Validating config")
+
+        valid_dict = {
+            "enable": False,
+            "havocType": False,
+            "totalDurationInSec": False
+        }
+
+        try:
+            if havoc_conf.enable in [0, 1]:
+                valid_dict["enable"] = True
+
+            if havoc_conf.havocType in [1, 2, 3, 4, 5, 6, 7, 8]:
+                valid_dict["havocType"] = True
+
+            if isinstance(havoc_conf.totalDurationInSec, int):
+                valid_dict["totalDurationInSec"] = True
+
+            havoc_type = havoc_conf.havocType
+
+            if havoc_type in [1, 2]:
+                valid_dict["uRL"] = False
+                if len(havoc_conf.uRL) != 0:
+                    valid_dict["uRL"] = True
+
+            if havoc_type in [3, 4]:
+                valid_dict["hostname"] = False
+                if havoc_conf.backendNameMod != 0:
+                    if len(havoc_conf.hostname) != 0:
+                        valid_dict["hostname"] = True
+                else:
+                    valid_dict["hostname"] = True
+
+            if havoc_type in [5, 6]:
+                valid_dict["methodFQM"] = False
+                if len(havoc_conf.methodFQM) != 0:
+                    valid_dict["methodFQM"] = True
+
+            if havoc_type in [1, 3, 5]:
+                valid_dict["delayInSec"] = False
+                if isinstance(havoc_conf.delayInSec, int):
+                    valid_dict["delayInSec"] = True
+
+            if havoc_type in [7]:
+                valid_dict["leaksINMB"] = False
+                if isinstance(havoc_conf.leaksINMB, int):
+                    valid_dict["leaksINMB"] = True
+                valid_dict["memoryLeaksInSec"] = False
+                if isinstance(havoc_conf.memoryLeaksInSec, int):
+                    valid_dict["memoryLeaksInSec"] = True
+
+            is_valid = True
+            return_message = "Config Validated"
+
+            for k in valid_dict:
+                if not valid_dict[k]:
+                    is_valid = False
+                    return_message = str(k) + " invalid"
+                    break
+
+            return is_valid, return_message
+
+        except Exception as e:
+            self.logger.debug(e)
+            return False, e
+
     # def parse_nethavoc_config(self, json_conf, havoc_type, header_dict):
     def parse_nethavoc_config(self, json_conf, header_dict):
         self.logger.info("Parsing config")
@@ -252,8 +335,11 @@ class NDNetHavocMonitor(object):
                 # self.netHavocConfigMap[havoc_type].append(req_obj)
 
             else:
-                self.logger.warning("Invalid config: {}".format(ret_message))
+                self.logger.info("Invalid config: {}".format(ret_message))
 
+            if havoc_type == 7:
+                self.increase_memory_usage()
+        
         except Exception as e:
             raise e
 
@@ -269,6 +355,7 @@ class NDNetHavocMonitor(object):
     def common_failure(self, exception_message):
         self.logger.info("Applying failure message: {}".format(exception_message))
 
+
         raise NDHavocException(exception_message)
 
     @staticmethod
@@ -297,18 +384,33 @@ class NDNetHavocMonitor(object):
 
         if config_list:
             for config in config_list:
+
+                if config.applying:
+                    self.logger.debug("Already applying, not proceeding further")
+                    return
+                config.applying = True
+
                 url = config.uRL
                 delay_in_second = config.delayInSec
 
-                # if url == req_url:
-                if url in req_url:
-                    self.logger.debug("Config matched for Request URL {}".format(req_url))
+                apply_all = False
+                if hasattr(config, 'bTNameMode'):
+                    if config.bTNameMode == 0:
+                        apply_all = True
+
+                if apply_all:
+                    self.logger.debug("bTNameMode = 0, applying")
                     self.common_delay_in_response(delay_in_second)
                 else:
-                    self.logger.debug("Config not matched for Request URL {}".format(req_url))
+                    if url in req_url:
+                        self.logger.debug("Config matched for Request URL {}".format(req_url))
+                        self.common_delay_in_response(delay_in_second)
+                    else:
+                        self.logger.debug("Config not matched for Request URL {}".format(req_url))
+                config.applying = False
+
         else:
-            self.logger.warning("config not set for apply_inbound_service_delay")
-            # raise Exception("config not set for apply_inbound_service_delay")
+            self.logger.debug("config not set for apply_inbound_service_delay")
 
     def apply_inbound_service_failure(self, req_url):
         self.logger.info("applying inbound service failure")
@@ -316,39 +418,68 @@ class NDNetHavocMonitor(object):
 
         if config_list:
             for config in config_list:
+                if config.applying:
+                    self.logger.debug("Already applying, not proceeding further")
+                    return
+                config.applying = True
+
                 url = config.uRL
 
-                # if url == req_url:
-                if url in req_url:
-                    self.logger.debug("Config matched for Request URL {}".format(req_url))
-                    self.common_failure(url)  # CHECK ARGUMENT
+                apply_all = False
+                if hasattr(config, 'bTNameMode'):
+                    if config.bTNameMode == 0:
+                        apply_all = True
+
+                if apply_all:
+                    self.logger.debug("bTNameMode = 0, applying")
+                    config.applying = False
+                    self.common_failure(url)
                 else:
-                    self.logger.debug("Config not matched for Request URL {}".format(req_url))
+                    if url in req_url:
+                        self.logger.debug("Config matched for Request URL {}".format(req_url))
+                        config.applying = False
+                        self.common_failure(url)
+                    else:
+                        self.logger.debug("Config not matched for Request URL {}".format(req_url))
+                        config.applying = False
+
+
 
         else:
-            self.logger.warning("config not set for apply_inbound_service_failure")
+            self.logger.debug("config not set for apply_inbound_service_failure")
             # raise Exception("config not set for apply_inbound_service_failure")
 
-        pass
-
     def apply_outbound_service_delay(self, req_hostname):
         self.logger.info("applying outbound service delay")
         config_list = self.netHavocConfigMap["OutBoundService_Delay"]
 
         if config_list:
             for config in config_list:
+                if config.applying:
+                    self.logger.debug("Already applying, not proceeding further")
+                    return
+
+                config.applying = True
                 hostname = config.hostname
                 delay_in_second = config.delayInSec
 
-                # if hostname == req_hostname:
-                if hostname in req_hostname:
+                apply_all = False
+                if hasattr(config, 'backendNameMod'):
+                    if config.backendNameMod == 0:
+                        apply_all = True
 
-                    self.logger.debug("Config matched for Request Hostname {}".format(req_hostname))
+                if apply_all:
+                    self.logger.debug("backendNameMod = 0, applying")
                     self.common_delay_in_response(delay_in_second)
                 else:
-                    self.logger.debug("Config not matched for Request Hostname {}".format(req_hostname))
+                    if hostname in req_hostname:
+                        self.logger.debug("Config matched for Request Hostname {}".format(req_hostname))
+                        self.common_delay_in_response(delay_in_second)
+                    else:
+                        self.logger.debug("Config not matched for Request Hostname {}".format(req_hostname))
+                config.applying = False
         else:
-            self.logger.warning("config not set for apply_outbound_service_delay")
+            self.logger.debug("config not set for apply_outbound_service_delay")
             # raise Exception("config not set for apply_outbound_service_delay")
 
     def apply_outbound_service_failure(self, req_hostname):
@@ -357,18 +488,33 @@ class NDNetHavocMonitor(object):
 
         if config_list:
             for config in config_list:
+                if config.applying:
+                    self.logger.debug("Already applying, not proceeding further")
+                    return
+                config.applying = True
+
                 hostname = config.hostname
 
-                # if hostname == req_hostname:
-                if hostname in req_hostname:
+                apply_all = False
+                if hasattr(config, 'backendNameMod'):
+                    if config.backendNameMod == 0:
+                        apply_all = True
 
-                    self.logger.debug("Config matched for Request Hostname {}".format(req_hostname))
-                    self.common_failure(hostname)  # CHECK ARGUMENT
+                if apply_all:
+                    self.logger.debug("backendNameMod = 0, applying")
+                    config.applying = False
+                    self.self.common_failure(hostname)
                 else:
-                    self.logger.debug("Config not matched for Request Hostname {}".format(req_hostname))
+                    if hostname in req_hostname:
+                        self.logger.debug("Config matched for Request Hostname {}".format(req_hostname))
+                        config.applying = False
+                        self.common_failure(hostname)
+                    else:
+                        self.logger.debug("Config not matched for Request Hostname {}".format(req_hostname))
+                        config.applying = False
 
         else:
-            self.logger.warning("config not set for apply_outbound_service_failure")
+            self.logger.debug("config not set for apply_outbound_service_failure")
             # raise Exception("config not set for apply_outbound_service_failure")
 
     def apply_method_call_delay(self, req_method_fqm):
@@ -377,19 +523,22 @@ class NDNetHavocMonitor(object):
 
         if config_list:
             for config in config_list:
+                if config.applying:
+                    self.logger.debug("Already applying, not proceeding further")
+                    return
+                config.applying = True
+
                 delay_in_second = config.delayInSec
                 method_fqm = config.methodFQM
 
-                print("\n\nmethod_fqm == req_method_fqm", method_fqm, req_method_fqm)
-
-                # if method_fqm == req_method_fqm:
                 if method_fqm in req_method_fqm:
                     self.logger.debug("Config matched for Request FQM {}".format(req_method_fqm))
                     self.common_delay_in_response(delay_in_second)
                 else:
                     self.logger.debug("Config not matched for Request FQM {}".format(req_method_fqm))
+                config.applying = False
         else:
-            self.logger.warning("config not set for apply_method_call_delay")
+            self.logger.debug("config not set for apply_method_call_delay")
             # raise Exception("config not set for apply_method_call_delay")
 
     def apply_method_invocation_failure(self, req_method_fqm):
@@ -398,20 +547,43 @@ class NDNetHavocMonitor(object):
 
         if config_list:
             for config in config_list:
-                method_fqm = config.methodFQM
+                if config.applying:
+                    self.logger.debug("Already applying, not proceeding further")
+                    return
+                config.applying = True
 
+                method_fqm = config.methodFQM
                 # if method_fqm == req_method_fqm:
                 if method_fqm in req_method_fqm:
                     self.logger.debug("Config matched for Request FQM {}".format(req_method_fqm))
+                    config.applying = False
                     self.common_failure(method_fqm)
                 else:
                     self.logger.debug("Config not matched for Request FQM {}".format(req_method_fqm))
+                    config.applying = False
+
         else:
-            self.logger.warning("config not set for apply_method_call_failure")
+            self.logger.debug("config not set for apply_method_call_failure")
             # raise Exception("config not set for apply_method_call_failure")
 
     def increase_memory_usage(self):
-        pass
+        self.logger.info("applying memory leak")
+        config_list = self.netHavocConfigMap["Custom_Memory_Leak"]
+        if config_list:
+            for config in config_list:
+                if config.applying:
+                    self.logger.debug("Already applying, not proceeding further")
+                    return
+                config.applying = True
+
+                leaksINMB = config.leaksINMB
+                totalDurationInSec = config.totalDurationInSec
+                memoryLeaksInSec = config.memoryLeaksInSec
+                apply_memory_leak(leaksINMB, totalDurationInSec, memoryLeaksInSec)
+                config.applying = False
+
+        else:
+            self.logger.debug("config not set for apply_method_call_failure")
 
     def increase_cpu_utilization(self):
         pass
@@ -450,7 +622,7 @@ class NDNetHavocMonitor(object):
                     else:
                         self.logger.debug("Config {} for type {} not expired, continuing".format(id(config), config.havocType))
             else:
-                self.logger.warning("config not found for havoc type {}".format(havoc_type))
+                self.logger.debug("config not found for havoc type {}".format(havoc_type))
 
 
 if __name__ == '__main__':
